#!/usw/tcl8.0/bin/wish
# Tcl-O-Scope - A program to monitor the size of global variables.
# Author:	C.Rajkumar Madhuram
#			madhurrc@orl.wec.com
#
# Modifications and distribution of this program are subject to the laws of freeware. The author holds
# no responsibility for ANY direct or indirect consequences that might arise from the use of this program. 
#

package provide scope 1.0
catch {package require Blt}
package require Tclx
package require Tix

namespace eval scope {
	variable SC
	
	proc getVarInfo {} {
		set glVars [info globals]
		set nScalars 0
		set nArray 0
		set nElems 0
		foreach var $glVars {
			global $var
			set names [array names $var] 
			if {$names == ""} {
				incr nScalars
				incr nElems
			} else {
				incr nArray
				incr nElems [llength $names]
			}
		}
		return [list $nScalars $nArray $nElems]
	}
		
	proc sampleParams {} {
		variable SC
		variable xVector 
		variable yVector1
		variable yVector2 
		variable yVector3
		
		lassign [getVarInfo] nScalars nArray nElems
		incr SC(counter)
		set ctr $SC(counter)
		set hlist $SC(win.hlist)
		
		if [cequal $SC(choice) auto] {
			set txt " Read_$ctr  "
		} else {
			set desc [$SC(win.desc) subwidget entry get]
			set lb [$SC(win.desc) subwidget listbox]
			set list [$lb get 0 end]
			if {([lsearch $list $desc] == -1) && ($desc != "")} {
				$lb insert end $desc
			}
			set txt " $desc  "
		}
		
		$hlist add index_$SC(counter) -itemtype text -text $ctr
		$hlist item create index_$ctr 1 -itemtype text -text $txt
		$hlist item create index_$ctr 2 -itemtype text -text $nScalars
		$hlist item create index_$ctr 3 -itemtype text -text $nArray
		$hlist item create index_$ctr 4 -itemtype text -text $nElems
		
		$hlist see index_$ctr
		if [cequal $SC(choice) auto] {
			set SC(afterId) [after $SC(time) scope::sampleParams]
		}
		
		
		set xVector(++end) $ctr
		set yVector1(++end) $nScalars 
		set yVector2(++end) $nArray
		set yVector3(++end) $nElems
	}
	
	# called whenever the choice (periodic/action based) is altered.
	proc changeChoice { args } {
		variable SC
		
		switch $SC(choice) {
			auto {
				foreach x {interval resume} { $SC(win.$x) config -state normal }
				foreach x {desc stop grab} { $SC(win.$x) config -state disabled }
			}
			
			manual {
				foreach x {interval stop resume} { $SC(win.$x) config -state disabled }
				foreach x {desc grab} { $SC(win.$x) config -state normal }
			}
		}
	}
	
	proc startAuto {} {
		variable SC
		
		$SC(win.stop) config -state normal
		$SC(win.resume) config -state disabled
		sampleParams
		
	}
	
	proc stopAuto {} {
		variable SC
		
		$SC(win.stop) config -state disabled
		$SC(win.resume) config -state normal
		after cancel $SC(afterId)
	}
	
	proc selectShow { type } {
		variable SC
		
		$SC(graph) element config $type -mapped $SC(check.$type)
	}
	
	proc close {} {
		variable SC
	
		destroy $SC(w)
		namespace delete ::scope
	}
	
	proc clearAll {} {
		variable SC
	
		variable xVector 
		variable yVector1
		variable yVector2 
		variable yVector3
		
		set SC(counter) 0
		$SC(win.hlist) delete all
		unset xVector yVector1 yVector2 yVector3
		vector xVector(0) yVector1(0) yVector2(0) yVector3(0)
		set grf $SC(graph)
		$grf element config sc -xdata xVector -ydata yVector1 -color red
		$grf element config ar -xdata xVector -ydata yVector2 -color blue 
		$grf element config el -xdata xVector -ydata yVector3 -color green
	}
	
	proc init {} {
		variable SC 
		variable xVector 
		variable yVector1
		variable yVector2 
		variable yVector3
		
		set size [set SC(size) 100]
		
		::vector xVector(0) yVector1(0) yVector2(0) yVector3(0)
		set SC(counter) 0
		set SC(w) [set w [toplevel .scope]]
		wm title $w "Tcl-o-Scope"
		wm protocol $w WM_DELETE_WINDOW scope::close
		set SC(font) {helvetica 9}
		
		set f1 [frame $w.down]
			grid columnconfig $f1 2 -weight 1
			grid columnconfig $f1 3 -weight 1
			grid [button $f1.b1 -text "Clear" -command scope::clearAll] -row 0 -column 0 -rowspan 2 -ipadx 5 -ipady 5 -sticky news
			grid [radiobutton $f1.rb1 -text "Periodic" -variable scope::SC(choice) -value auto -command scope::changeChoice ] -row 0 -column 1 -sticky w
			grid [radiobutton $f1.rb2 -text "Action Based" -variable scope::SC(choice) -value manual -command scope::changeChoice ] -row 1 -column 1 -sticky w
			grid [tixLabelEntry $f1.le1 -label "Interval (ms):" ] -row 0 -column 2 -columnspan 2 -sticky ew
			grid [button $f1.b2 -text "Stop" -command "scope::stopAuto"] -row 1 -column 2  -sticky news
			grid [button $f1.b3 -text "Start" -command "scope::startAuto"] -row 1 -column 3  -sticky news
			
			$f1.le1 subwidget entry config -textvariable scope::SC(time) 
			set SC(time) 1000
			set SC(win.interval) $f1.le1 
			set SC(win.stop) $f1.b2
			set SC(win.resume) $f1.b3
			foreach x {interval stop resume} { $SC(win.$x) config -state disabled }
			
		pack $f1 -side bottom -fill x -expand n
		
		set SC(choice) manual
		set sep [frame $w.sep -relief ridge -bd 2 -height 3]
		pack $sep -side bottom -fill x
		
		set f_rt [frame $w.right] 
			set top [::tixLabelFrame $f_rt.top -label "Display"]
			set frm [$top subwidget frame]
			set grf [set SC(graph) [graph $frm.grf -width 300 -height 200 -bg #bdc9d8 -plotbackground white -title ""]]
			$grf legend conf -mapped no
			$grf xaxis conf -title "Samples" -titlefont $SC(font) -tickfont $SC(font) 
			$grf yaxis conf -title "Count" -titlefont $SC(font) -tickfont $SC(font)
			$grf grid conf -mapped yes
			$grf element create sc -xdata xVector -ydata yVector1 -color red
			$grf element create ar -xdata xVector -ydata yVector2 -color blue 
			$grf element create el -xdata xVector -ydata yVector3 -color green 
			pack $grf -side top -fill both -expand y -padx 5 -pady 5
			
			set frm1 [frame $frm.bot]
			pack [checkbutton $frm1.cb1 -text "Scalars" -font $SC(font) -variable scope::SC(check.sc) -command "scope::selectShow sc"] -side left
			pack [checkbutton $frm1.cb2 -text "Arrays" -font $SC(font) -variable scope::SC(check.ar) -command "scope::selectShow ar"] -side left
			pack [checkbutton $frm1.cb3 -text "Elements" -font $SC(font) -variable scope::SC(check.el) -command "scope::selectShow el"] -side left
			
			foreach type {sc ar el} { set SC(check.$type) 1 }
			pack $frm1 -side top -fill x 
			pack $top -side top -fill both -expand y 
			
			::Blt_ZoomStack $grf
			
			set bot [::tixLabelFrame $f_rt.bot -label "Data"]
			set frm [$bot subwidget frame]
			
			::tixScrolledHList $frm.shl -height 1.2i -options {
				hlist.columns 5
				hlist.header true
				vsb.width 10
				hsb.width 10
			}
			
			set hlist [$frm.shl subwidget hlist]
			set SC(win.hlist) $hlist
			$hlist configure -font {helvetica 9}
			set bcolor #bbbbbb
			$hlist header create 0 -itemtype text -text "S.No" -headerbackground $bcolor		
			$hlist header create 1 -itemtype text -text "Item"  -headerbackground $bcolor
			$hlist header create 2 -itemtype text -text "Scalars"  -headerbackground $bcolor
			$hlist header create 3 -itemtype text -text "Arrays" -headerbackground $bcolor
			$hlist header create 4 -itemtype text -text "Elements" -headerbackground $bcolor

			$hlist column width 0 -char 5
			$hlist column width 1 {}
			$hlist column width 2 -char 10
			$hlist column width 3 -char 10
			$hlist column width 4 -char 10
			
			pack $frm.shl -side top -fill both -expand y
			
			set frm1 [frame $frm.bot]
				::tixComboBox $frm1.cb1 -label "Description: " -editable true
				button $frm1.b1 -text "Sample" -command "scope::sampleParams"
				pack $frm1.cb1  -side left -fill x -expand y
				pack $frm1.b1 -side right -padx 5
				set SC(win.desc) $frm1.cb1
				set SC(win.grab) $frm1.b1
			pack $frm1 -side top -fill x
			pack $bot -side top -fill both -expand n 
			
		pack $f_rt -side right -fill both -expand y		
		
		set SC(count) 0
		set SC(stopFlag) 0
	}		
}

scope::init
